import { SIDEBAR_HEIGHT, SIDEBAR_WIDTH, TILE_SIZE } from "./constants";
import { HeldPiece } from "./heldPiece";
import { addPoint, minMax, pickRandom } from "./lib/util";
import { TETRIS_PIECE, TETRIS_PIECE_LIST } from "./tetrisPieces";
import { TextureStore } from "./textureStore";

const DIGIT_HEIGHT = 8;
const DIGIT_WIDTH = 6;

export class Sidebar {
	private scoreNum: number;
	private nextPiece: HeldPiece;
	reDraw: boolean;
	constructor() {
		this.scoreNum = 0;
		this.reDraw = true;
		this.nextPiece = { schema: TETRIS_PIECE.O, pos: [0, 0], tiles: [] };
		this.getNextPiece();
	}
	getNextPiece(): HeldPiece {
		const old = this.nextPiece;
		const type = pickRandom(TETRIS_PIECE_LIST);
		this.nextPiece = {
			schema: type,
			pos: [4, 0],
			tiles: structuredClone(type.tiles),
		};
		this.reDraw = true;
		return old;
	}
	set score(s: number) {
		this.scoreNum = s;
		this.reDraw = true;
	}

	get score(): number {
		return this.scoreNum;
	}
	draw(ctx: CanvasRenderingContext2D, textures: TextureStore): void {
		ctx.drawImage(textures.scoreboardBg, 0, 0, SIDEBAR_WIDTH, SIDEBAR_HEIGHT);
		// Draw Next Up tetrimino
		const tex = textures.tile[this.nextPiece.schema.color];
		const xOffsets = this.nextPiece.tiles.map((p) => p[0]);
		const [smallest, biggest] = minMax(xOffsets);
		const width = (biggest - smallest + 1) * TILE_SIZE;
		const widthOffset = (SIDEBAR_WIDTH - width) / 2;
		for (const tile of this.nextPiece.tiles) {
			ctx.drawImage(
				tex,
				(tile[0] - smallest) * TILE_SIZE + widthOffset,
				(tile[1] + 3.75) * TILE_SIZE,
				TILE_SIZE,
				TILE_SIZE
			);
		}
		// Draw score
		const digits = this.scoreNum.toString().split("");
		const digitsWidth = digits.length * DIGIT_WIDTH;
		const offset = (SIDEBAR_WIDTH - digitsWidth * 2) / 2;
		for (const [i, digit] of digits.entries()) {
			ctx.drawImage(
				textures.numbers[Number.parseInt(digit)],
				2 * DIGIT_WIDTH * i + offset,
				16 * 2,
				DIGIT_WIDTH * 2,
				DIGIT_HEIGHT * 2
			);
		}

		this.reDraw = false;
	}
	reset(): void {
		this.scoreNum = 0;
		this.reDraw = true;
		this.getNextPiece();
	}
}
